==> kotobase-0.2.1-py3-none-any.whl: kotobase/api.py <==
from __future__ import annotations

import concurrent.futures as _cf
from functools import lru_cache
from typing import (List,
                    Dict,
                    Optional)

from kotobase.core.datatypes import (
    LookupResult,
    KanjiDTO,
    JMDictEntryDTO,
    JMNeDictEntryDTO,
    SentenceDTO
    )
import re
from kotobase.db_builder.config import DB_BUILD_LOG_PATH
from kotobase.repos.jmdict import JMDictRepo
from kotobase.repos.jmnedict import JMNeDictRepo
from kotobase.repos.kanji import KanjiRepo
from kotobase.repos.jlpt import JLPTRepo
from kotobase.repos.sentences import SentenceRepo


class Kotobase:
    """
    Stateless class that orchestrates the individual repositories and
    returns rich, serialisable objects.
    """
    # Core
    def lookup(
        self,
        word: str,
        *,
        wildcard: bool = False,
        include_names: bool = False,
        sentence_limit: int = 50,
    ) -> LookupResult:
        """
        Comprehensive word lookup.

        Parameters
        ----------
        word : str
            The query string (kana, kanji).
            Supports SQL wildcards '*' or '%'.
        wildcard : bool, default False
            If True, passes wildcards through unchanged.  If False,
            the search is exact (JMdict) but Tatoeba uses `%word%`
            containment.
        include_names : bool, default False
            Also query JMnedict (proper names).  Can be slow on very
            broad wildcards.
        sentence_limit : int, default 50
            Maximum number of Tatoeba sentences to fetch.

        Returns
        -------
        LookupResult
        """

        # Extract unique kanji found in the query
        kanji_chars = [c for c in word if "\u4e00" <= c <= "\u9fff"]
        entries: List[JMDictEntryDTO | JMNeDictEntryDTO] = []
        # Parallel  Lookups
        with _cf.ThreadPoolExecutor(max_workers=7) as pool:
            f_jmdict = pool.submit(JMDictRepo.search_form,
                                   word,
                                   limit=None if wildcard else 50)
            f_jmnedict = pool.submit(JMNeDictRepo.search,
                                     word,
                                     limit=50) if include_names else None
            f_kanji = pool.submit(KanjiRepo.bulk_fetch, kanji_chars)
            f_vocab = pool.submit(JLPTRepo.vocab_by_word, word)
            f_levels = pool.submit(JLPTRepo.kanji_levels, kanji_chars)
            f_grammar = pool.submit(JLPTRepo.grammar_entries_like, word)
            f_sent = pool.submit(
                SentenceRepo.search_containing,
                word,
                limit=sentence_limit,
                wildcard=wildcard,
            )
            # Retrieve results
            entries.extend(f_jmdict.result())
            if f_jmnedict:
                entries.extend(f_jmnedict.result())

            kanji_info: List[KanjiDTO] = f_kanji.result()
            jlpt_vocab = f_vocab.result()
            jlpt_kanji_levels: Dict[str, int] = f_levels.result()
            jlpt_grammar = f_grammar.result()
            sentences = f_sent.result()

        # 4. Aggregate
        return LookupResult(
            word=word,
            entries=entries,
            kanji=kanji_info,
            jlpt_vocab=jlpt_vocab,
            jlpt_kanji_levels=jlpt_kanji_levels,
            jlpt_grammar=jlpt_grammar,
            examples=sentences,
        )

    # Get Database Info From Build Log
    @staticmethod
    def db_info() -> Dict[str, str]:
        """
        Return a dictionary containing variables from
        Database build log file, if it exists.
        Raises EnvironmentError if file is not found.
        """
        if not DB_BUILD_LOG_PATH.exists():
            raise EnvironmentError("Database Build Log File Not Found")
        raw_log = DB_BUILD_LOG_PATH.read_text()
        info = {}
        matches = {
            "build_date": re.search(r'^BUILD_DATE=(.+)$',
                                    raw_log,
                                    re.MULTILINE),
            "build_time": re.search(r'^BUILD_TIME=(.+)$',
                                    raw_log,
                                    re.MULTILINE),
            "size_mb": re.search(r'^SIZE_MB=(.+)$',
                                 raw_log,
                                 re.MULTILINE)
            }
        for k, v in matches.items():
            if v:
                info[k] = v.group(1)
            else:
                info[k] = "N/A"
        return info

    # Convenience Wrappers

    @staticmethod
    @lru_cache(maxsize=10_000)
    def kanji(literal: str) -> Optional[KanjiDTO]:
        """Return a single KanjiDTO (or None)."""
        return KanjiRepo.by_literal(literal)

    @staticmethod
    @lru_cache(maxsize=20_000)
    def jlpt_level(word: str) -> Optional[int]:
        """Shortcut – just return JLPT vocab level for a word."""
        dto = JLPTRepo.vocab_by_word(word)
        return dto.level if dto else None

    @staticmethod
    def sentences(text: str, *, limit: int = 20) -> List[SentenceDTO]:
        """Fetch Japanese Tatoeba sentences containing *text*."""
        return SentenceRepo.search_containing(text, limit=limit)

    def __call__(self, word: str, **kwargs) -> LookupResult:
        """Alias for `lookup` so you can `Kotobase()(word)`."""
        return self.lookup(word, **kwargs)

    # Context Manager
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, tb):
        # Propagate Exceptions
        return False


__all__ = ["Kotobase"]

==> kotobase-0.2.2-py3-none-any.whl: kotobase/api.py <==
from __future__ import annotations

import concurrent.futures as _cf
from functools import lru_cache
from typing import (List,
                    Dict,
                    Optional)

from kotobase.core.datatypes import (
    LookupResult,
    KanjiDTO,
    JMDictEntryDTO,
    JMNeDictEntryDTO,
    SentenceDTO
    )
import re
from kotobase.db_builder.config import DB_BUILD_LOG_PATH
from kotobase.repos.jmdict import JMDictRepo
from kotobase.repos.jmnedict import JMNeDictRepo
from kotobase.repos.kanji import KanjiRepo
from kotobase.repos.jlpt import JLPTRepo
from kotobase.repos.sentences import SentenceRepo


class Kotobase:
    """
    Stateless class that orchestrates the individual repositories and
    returns serialisable objects.
    """
    # Core
    def lookup(
        self,
        word: str,
        *,
        wildcard: bool = False,
        include_names: bool = False,
        sentence_limit: int = 50,
    ) -> LookupResult:
        """
        Comprehensive word lookup.

        Args:
            word: The query string (kana, kanji)
                  Supports SQL wildcards '*' or '%'.

            wildcard: If True, passes wildcards through unchanged.  If False,
                      the search is exact (JMdict) but Tatoeba uses `%word%`
                      containment.

            include_names: Also query JMnedict (proper names).
                           Can be slow on very broad wildcards.

            sentence_limit: Maximum number of Tatoeba sentences to fetch.

        Returns:
            LookupResult Object.
        """

        # Extract unique kanji found in the query
        kanji_chars = [c for c in word if "\u4e00" <= c <= "\u9fff"]
        entries: List[JMDictEntryDTO | JMNeDictEntryDTO] = []
        # Parallel  Lookups
        with _cf.ThreadPoolExecutor(max_workers=7) as pool:
            f_jmdict = pool.submit(JMDictRepo.search_form,
                                   word,
                                   limit=None if wildcard else 50)
            f_jmnedict = pool.submit(JMNeDictRepo.search,
                                     word,
                                     limit=50) if include_names else None
            f_kanji = pool.submit(KanjiRepo.bulk_fetch, kanji_chars)
            f_vocab = pool.submit(JLPTRepo.vocab_by_word, word)
            f_levels = pool.submit(JLPTRepo.kanji_levels, kanji_chars)
            f_grammar = pool.submit(JLPTRepo.grammar_entries_like, word)
            f_sent = pool.submit(
                SentenceRepo.search_containing,
                word,
                limit=sentence_limit,
                wildcard=wildcard,
            )
            # Retrieve results
            entries.extend(f_jmdict.result())
            if f_jmnedict:
                entries.extend(f_jmnedict.result())

            kanji_info: List[KanjiDTO] = f_kanji.result()
            jlpt_vocab = f_vocab.result()
            jlpt_kanji_levels: Dict[str, int] = f_levels.result()
            jlpt_grammar = f_grammar.result()
            sentences = f_sent.result()

        # 4. Aggregate
        return LookupResult(
            word=word,
            entries=entries,
            kanji=kanji_info,
            jlpt_vocab=jlpt_vocab,
            jlpt_kanji_levels=jlpt_kanji_levels,
            jlpt_grammar=jlpt_grammar,
            examples=sentences,
        )

    # Get Database Info From Build Log
    @staticmethod
    def db_info() -> Dict[str, str]:
        """
        Return a dictionary containing variables from
        Database build log file, if it exists.

        Raises:
          EnvironmentError: If file doeesn't exist.
        """
        if not DB_BUILD_LOG_PATH.exists():
            raise EnvironmentError("Database Build Log File Not Found")
        raw_log = DB_BUILD_LOG_PATH.read_text()
        info = {}
        matches = {
            "build_date": re.search(r'^BUILD_DATE=(.+)$',
                                    raw_log,
                                    re.MULTILINE),
            "build_time": re.search(r'^BUILD_TIME=(.+)$',
                                    raw_log,
                                    re.MULTILINE),
            "size_mb": re.search(r'^SIZE_MB=(.+)$',
                                 raw_log,
                                 re.MULTILINE)
            }
        for k, v in matches.items():
            if v:
                info[k] = v.group(1)
            else:
                info[k] = "N/A"
        return info

    # Convenience Wrappers

    @staticmethod
    @lru_cache(maxsize=10_000)
    def kanji(literal: str) -> Optional[KanjiDTO]:
        """
        Return a single KanjiDTO (or None).

        Args:
          literal: Kanji Literal String.
        """
        return KanjiRepo.by_literal(literal)

    @staticmethod
    @lru_cache(maxsize=20_000)
    def jlpt_level(word: str) -> Optional[int]:
        """
        Shortcut – just return JLPT vocab level for a word.

        Args:
          word: Word to search for.
        """
        dto = JLPTRepo.vocab_by_word(word)
        return dto.level if dto else None

    @staticmethod
    def sentences(text: str, *, limit: int = 20) -> List[SentenceDTO]:
        """
        Fetch Japanese Tatoeba sentences containing *text*.

        Args:
          text: String text to search for
          limit: How many sentences to return.
        """
        return SentenceRepo.search_containing(text, limit=limit)

    def __call__(self, word: str, **kwargs) -> LookupResult:
        """
        Alias for `lookup` method
        """
        return self.lookup(word, **kwargs)

    # Context Manager
    def __enter__(self):
        """
        Context Manager entry point.
        """
        return self

    def __exit__(self, exc_type, exc, tb):
        # Propagate Exceptions
        """
        Propage exceptions in Context Manager close.
        """
        return False


__all__ = ["Kotobase"]
